package production

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/production/model"
)

type task struct {
	Table
	Case       Case
	order      string `data:"3"`
	fields     string `data:"title,content,task_status,case_count,begin_time,end_time,process_time,finish_time,issue_uid,plan_uid,user_uid"`
	create     string `name:"create title* content* begin_time:select@date end_time:select@date" role:"leader,worker"`
	modify     string `name:"modify title* content* begin_time*:select@date end_time*:select@date" role:"leader,worker"`
	remove     string `name:"remove" role:"leader,worker"`
	process    string `name:"process" role:"leader,worker"`
	finish     string `name:"finish" role:"leader,worker"`
	planBind   string `name:"planBind plan_uid*" role:"leader"`
	caseCreate string `name:"caseCreate title* content*" role:"leader,worker"`
}

func (s task) Create(m *ice.Message, arg ...string) {
	s.ValueCreate(m, kit.ArgDef(arg, kit.Simple(model.BEGIN_TIME, m.Time(), model.END_TIME, m.Time("72h"))...)...)
	s.issueCount(m, "1")
	s.planCount(m, "1")
	s.DashboardUpdate(m)
	s.GetCommandUID(m)
	s.SendMessage(m, "", "")
}
func (s task) Remove(m *ice.Message, arg ...string) {
	s.ValueRemove(m, arg...)
	s.issueCount(m, "-1")
	s.planCount(m, "-1")
	s.DashboardUpdate(m)
}
func (s task) List(m *ice.Message, arg ...string) {
	isLeader, user_uid := s.IsLeader(m), m.Option(model.USER_UID)
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		switch TaskStatus(kit.Int(value[model.TASK_STATUS])) {
		case TaskCreate:
			if isLeader && value[model.PLAN_UID] == "" {
				button = append(button, s.PlanBind)
			}
			if isLeader || user_uid == value[model.USER_UID] {
				button = append(button, s.Process)
			}
		case TaskProcess:
			button = append(button, s.CaseCreate)
			if isLeader || user_uid == value[model.USER_UID] {
				button = append(button, s.Finish)
			}
		case TaskFinish:
			kit.If(len(arg) == 2, func() { s.DoneMessage(m) })
		}
		m.PushButton(button...)
	}).Display("")
	s.SelectJoinPlan(m)
}
func (s task) Process(m *ice.Message, arg ...string) {
	s.changeStatus(m, TaskCreate, TaskProcess)
}
func (s task) Finish(m *ice.Message, arg ...string) {
	count := m.Cmd(s.Case, s.Select, "task_uid = ? AND status != ?", m.Option(model.UID), CaseFinish).Length()
	if m.WarnNotValid(count > 0, kit.Format("还有 %v 个未完成的用例", count)) {
		return
	}
	s.changeStatus(m, TaskProcess, TaskFinish)
	s.DashboardUpdate(m)
}
func (s task) PlanBind(m *ice.Message, arg ...string) {
	s.ValueModify(m, arg...)
	s.planCount(m, "1")
}
func (s task) CaseCreate(m *ice.Message, arg ...string) {
	m.Cmd(s.Case, s.Create, arg, model.TASK_UID, m.Option(model.UID), m.OptionSimple(model.PLAN_UID))
}

func init() { ice.TeamCtxCmd(task{}) }

func (s task) changeStatus(m *ice.Message, from, to TaskStatus) {
	s.ChangeStatus(m, int(from), int(to), m.ActionKey()+"_time", m.Time())
}
func (s task) planCount(m *ice.Message, value string) {
	s.Table.planCount(m, model.TASK_COUNT, value)
}
func (s task) issueCount(m *ice.Message, value string) {
	if m.IsErr() || m.Option(model.ISSUE_UID) == "" {
		return
	}
	m.Cmd(issue{}, s.AddCount, model.TASK_COUNT, value, m.Option(model.ISSUE_UID))
}

type TaskStatus int

const (
	TaskCreate TaskStatus = iota
	TaskProcess
	TaskFinish
)

var TaskStatusList = map[TaskStatus]string{
	TaskCreate:  "create",
	TaskProcess: "process",
	TaskFinish:  "finish",
}

func (s TaskStatus) String() string { return TaskStatusList[s] }
